#!/usr/bin/env python

"""
This script will drive through the creation of the MySQL database where data
collected by the file integrity checker 'Spot The Difference' will be stored.
After the creation and population of the database with a privileged user, it is
strongly recommended to perform all later checks with an unprivileged user, with
only 'SELECT' granted.
"""

import getpass
from socket import gethostname
import MySQLdb
from sys import exit, exc_info
from traceback import format_exception


################################################################################
# Constants definition                                                         #
################################################################################

# Default settings
USER   = getpass.getuser()
HOST   = "localhost"
PORT   = "3306"
SOCKET = "/var/run/mysql/mysql.sock"
DB     = "Spot"
TABLE  = gethostname().replace(".", "_")  # dots are not allowed in table names


################################################################################
# Helper Functions                                                             #
################################################################################

def get_input(prompt, default=""):
    """Return user input or, if empty, the default value"""
    return raw_input("%s [%s] " % (prompt, default)) or default


def get_connection_params():
    """Get connection parameters from the user"""
    kwargs = {}

    print "You will now be prompted a few questions to connect to MySQL:"

    kwargs["user"] = get_input("User:", USER)
    kwargs["passwd"] = getpass.getpass()
    kwargs["host"] = get_input("Host:", HOST)
    ans = get_input("Connect through TCP port or Unix socket? (Port, Socket)",
                    "Port")

    if ans.lower().startswith("p"):
        kwargs["port"] = int(get_input("Port:", PORT))
    else:
        kwargs["unix_socket"] = get_input("Socket:", SOCKET)

    return kwargs


################################################################################
# Main function                                                                #
# It creates the database (if it doesn't exist yet) and the table where to     #
# store information about this host                                            #
################################################################################

def run():
    """Perform all the steps necessary for the creation of the database"""
    # A brief introduction
    print __doc__

    kwargs = get_connection_params()

    db_name = get_input("Database name:", DB)
    table = get_input("Table name:", TABLE)

    # Connect to the database
    print "\nConnecting to MySQL server on '%s'..." % kwargs["host"],
    try:
        db = apply(MySQLdb.connect, (), kwargs)
    except:
        print "failed"
        exit(format_exception(*exc_info())[-1].split('"')[1])
    else:
        print "ok"

    # Create the database
    try:
        db.query("CREATE DATABASE IF NOT EXISTS %s;" % db_name)
    except:
        print "\nFailed to create database '%s'" % db_name
        exit("Error: %d - %s" % (db.errno(), db.error()))
    else:
        print "Database '%s' successfully created (or already existed)" % \
              db_name

    # Select the database
    print "selecting database '%s'..." % db_name,
    try:
        db.select_db(db_name)
    except:
        print "failed"
        exit("Error: %d - %s" % (db.errno(), db.error()))
    else:
        print "ok"

    # Create the table
    print "Creating table '%s'..." % table,
    try:
        db.query("""CREATE TABLE %s (
                           path     VARCHAR(255) BINARY PRIMARY KEY,
                           md5      CHAR(32),
                           sha      CHAR(40),
                           st_mode  SMALLINT     UNSIGNED,
                           st_ino   MEDIUMINT    UNSIGNED,
                           st_dev   SMALLINT     UNSIGNED,
                           st_nlink SMALLINT     UNSIGNED,
                           st_uid   INT          UNSIGNED,
                           st_gid   INT          UNSIGNED,
                           st_size  BIGINT       UNSIGNED,
                           st_atime INT          UNSIGNED,
                           st_mtime INT          UNSIGNED,
                           st_ctime INT          UNSIGNED
                    );""" % table)
    except:
        print "failed"
        exit("Error %d - %s" % (db.errno(), db.error()))
    else:
        print "ok"

    # Close the connection to the database
    db.commit()
    db.close()

    # Print final recommendations
    print """
    +---------
    | Don't forget to add these entries to your configuration file:
    |
    | user = %s
    | passwd = <your_password>
    | host = %s""" % (kwargs["user"], kwargs["host"])

    if kwargs.has_key("port"):
        print "    | port =", kwargs["port"]
    else:
        print "    | unix_socket =", kwargs["unix_socket"]

    print """    | db = %s
    | table = %s
    +---------
    """ % (db_name, table)

    print "That's all!"

