#!/usr/bin/env python

"""
This script will drive through the creation of the PostgreSQL database where
data collected by the file integrity checker 'Spot The Difference' will be
stored.
After the creation and population of the database with a privileged user, it is
strongly recommended to perform all later checks with an unprivileged user, with
only 'SELECT' granted.
"""


import os
import psycopg
import getpass
from socket import gethostname
from traceback import format_exception
from sys import exit, exc_info


################################################################################
# Constants definition                                                         #
################################################################################

# Default settings
USER  = getpass.getuser()
HOST  = "/tmp"
PORT  = "5432"
DB    = "Spot"
TABLE = gethostname().replace(".", "_")  # dots not allowed in table names


################################################################################
# Helper functions                                                             #
################################################################################

def get_input(prompt, default=""):
    """Return user input or the default value"""
    return raw_input("%s [%s] " % (prompt, default)) or default


def create_db(kwargs, dbname):
    """ """
    # Connect to the default database 'template1'
    print "\nConnecting to PostgreSQL server '%s'..." % kwargs["host"],
    kwargs["database"] = "template1"
    try:
        db = apply(psycopg.connect, (), kwargs)
    except:
        print "failed"
        exit(format_exception(*exc_info())[-1])
    else:
        print "ok"

    db.autocommit()
    c = db.cursor()

    # create the database
    print "Creating database '%s'..." % dbname,
    try:
        c.execute("CREATE DATABASE %s;" % dbname)
    except:
        print "failed"
        exit(format_exception(*exc_info())[-1])
    else:
        print "ok"
        db.close()


def get_connection_params():
    """Get connection parameters from the user"""
    kwargs = {}

    print "You will now be prompted a few questions to connect to PostgreSQL:"

    kwargs["user"] = get_input("User:", USER)
    kwargs["password"] = getpass.getpass()
    kwargs["host"] = get_input("Host: (Can be a name, IP or directory)", HOST)

    if not os.path.isdir(kwargs["host"]):
        kwargs["port"] = get_input("Port:", PORT)

    ans = get_input("Create a new database?", "yes")
    dbname = get_input("Database name:", DB)
    table = get_input("Table name:", TABLE)
    kwargs["serialize"] = 1

    if ans.lower().startswith("y"):
        create_db(kwargs, dbname)

    kwargs["database"] = dbname
    return (kwargs, table)


################################################################################
# Main function                                                                #
# It creates the database (if it doesn't exist yet) and the table where to     #
# store information about this host                                            #
################################################################################

def run():
    """Perform all the steps necessary for the creation of the database"""
    # A brief introduction
    print __doc__

    kwargs, table = get_connection_params()

    # Connect to the database
    print "Connecting to database '%s'..." % kwargs["database"],
    try:
        db = apply(psycopg.connect, (), kwargs)
    except:
        print "failed"
        exit(format_exception(*exc_info())[-1])
    else:
        print "ok"

    # Create the table
    print "Creating table '%s'..." % table,
    c = db.cursor()
    try:
        c.execute("""CREATE TABLE %s (
                            path     VARCHAR(255) PRIMARY KEY,
                            md5      CHAR(32),
                            sha      CHAR(40),
                            st_mode  INT,
                            st_ino   INT,
                            st_dev   INT,
                            st_nlink INT,
                            st_uid   INT,
                            st_gid   INT,
                            st_size  BIGINT,
                            st_atime INT,
                            st_mtime INT,
                            st_ctime INT
                     );""" % table)
    except:
        print "failed"
        exit(format_exception(*exc_info())[-1])
    else:
        print "ok"

    # Close the connection to the database
    db.commit()
    db.close()

    # Final recommendations
    print """
    +---------
    | Don't forget to add these entries to your configuration file:
    |
    | user = %s
    | passwd = <your_password>
    | host = %s""" % (kwargs["user"], kwargs["host"])

    if kwargs.has_key("port"):
        print "    | port =", kwargs["port"]

    print """    | db = %s
    | table = %s
    +---------
    """ % (kwargs["database"], table)

    print "That's all!"

if __name__ == "__main__":
    run()
